/*
 * mf_stepper.c
 *
 * Created: 2013-06-10 08:36:41
 *  Author: dsosnowski
 */ 



#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "defs.h"


#define DEFAULT_PERIOD			64
#define ANALOG_PERIOD			16


void HardInit(void);
void AnalogProc(void);

struct {
	char delay;
	char move;
	char period;
	char status;
} analog;


struct {
	volatile char cycle;
	char main;
	volatile char tick;
} delay;

const char steps[] = {
	0b10011111,
	0b10101111,
	0b10101111,
	0b01101111,
	0b01101111,
	0b01011111,
	0b01011111,
	0b10011111
};

//rise fall
//---- ----
const char swim[] = {
	0b10001111,
	0b10001111,
	0b00101111,
	0b00101111,
	0b01001111,
	0b01001111,
	0b00011111,
	0b00011111
};

struct {
	volatile char step;
	char index;
	char direction;
	char period;	
	char status;
} stepper;

struct {
	volatile char step;
	char index;
	char direction;
	char period;	
} micro;	



//---------------------------------------------------
int main(void)
{
	char temp;
	
	HardInit();
	
	stepper.step = 0xFF;
	micro.period = DEFAULT_PERIOD;
	
    while(1)
    {
		//-----------------------------------
		//init timer tick
		TCCR0 = (1<<PSR0 | 1<<CS01);
		TCNT0 = analog.move;
		TIFR = (1<<TOV0);
		
		//-----------------------------------
		//execute status
		if (stepper.status == 0) 
		{
			micro.index++;
			if (GET_CFG_R6 == 0) micro.index += 7;
		}	
		else
		{
			if (GET_CFG_R5 == 0)
			{
				stepper.step = 0b00001111;
				micro.step = 0b00001111; 										
			}
		}
	
		
		//-----------------------------------
		//next step				
		if (micro.index >= micro.period) 
		{
			micro.index = 0;
			
			//next step
			if (stepper.direction == 0)
			{
				stepper.index++;
				if (stepper.index > 7) stepper.index = 0;				
			}
			else
			{
				stepper.index--;
				if (stepper.index > 7) stepper.index = 7;				
			}

			//update
			if (stepper.status == 0)
			{
				temp = swim[stepper.index];
				stepper.step = steps[stepper.index];
				micro.step = temp; 								
			}
		}
		else analog.period++;
		
		//-----------------------------------
		//delay calc
		if (delay.tick)
		{
			delay.tick = 0;
			if (delay.main & 1) LED_OFF;
			else LED_SET;
			if (delay.main) delay.main--;			
		}

		//-----------------------------------
		//analog periods
		if (analog.period >= ANALOG_PERIOD)
		{
			analog.period = 0;
			AnalogProc();
		}		
		
		//-----------------------------------
		//calc micro direction
		if (stepper.direction == 0)
		{
			if ((stepper.index & 1) == 0) 
				OCR1A = (micro.period - micro.index);		
			else
				OCR1A = micro.index;		
		}
		else
		{
			if ((stepper.index & 1) == 1) 
				OCR1A = (micro.period - micro.index);		
			else
				OCR1A = micro.index;							
		}
		
		//--------------------------------------
		//wait for timer tick		
		while ((TIFR & (1<<TOV0)) == 0);
		
    }
}


//---------------------------------------------------
void HardInit(void)
{
	OUT_PORT = (1<<CFG_R5 | 1<<CFG_R6);
	OUT_DDR = (1<<ENA_AB | 1<<ENA_CD | 1<<OUT_A | 1<<OUT_B | 1<<OUT_C | 1<<OUT_D);
	
	STEER_PORT = (1<<STEER_MOSI | 1<<STEER_MISO | 1<<STEER_SCK | 1<<STEER_STOP | 1<<STEER_RST);
	STEER_DDR = (1<<STEER_LED);	
	
	TCCR1A = (1<<PWM1A);
	TCCR1B = (1<<CTC1 | 5);			//16 divider
	
	OCR1C = DEFAULT_PERIOD;
	OCR1A = 0;
	
	TIMSK = (1<<OCIE1A | 1<<TOIE1);
	sei();
}

//---------------------------------------------------
void AnalogProc(void)
{
	static char cycle;
	//read ch7, start ch8
	if ((ADCSR & (1<<ADSC)) == 0)
	{	
		if (cycle == 0)
		{
			cycle = 1;	
			analog.delay = ADCH;
			ADMUX = (1<<ADLAR | 8);
			ADCSR = (1<<ADEN | 1<<ADSC | 6);
			if (analog.delay < 2) analog.delay = 1;
			
			stepper.status = 1;
			if (GET_STOP == 0)
			{
				delay.main = (analog.delay);				
				if (analog.status == 0) 
					stepper.status = 0;
			}
			else
			{
				if (delay.main)
				{
					if (analog.status == 0) 
						stepper.status = 0;
				}				
			} 
		}	
		else
		{
			cycle = 0;
			analog.move = ADCH;
			ADMUX = (1<<ADLAR | 7);
			ADCSR = (1<<ADEN | 1<<ADSC | 6);
			
			analog.status = 0;
			if (analog.move >= 138) 
			{
				stepper.direction = 1;
				analog.move -= 138;
			}			
			else
			{
				if (analog.move > 118) analog.status = 1;
				else
				{
					stepper.direction = 0;
					analog.move = (118 - analog.move);											
				}			
			}									
			analog.move *= 2;
		}
	}	
}

//---------------------------------------------------
ISR (TIMER1_CMPA_vect)
{
	OUT_PORT = micro.step; 
}

//---------------------------------------------------
ISR (TIMER1_OVF1_vect)
{
	OUT_PORT = stepper.step;
	delay.cycle++;
	if (delay.cycle == 0) delay.tick = 1; 
}


